#!/usr/bin/perl -w
#
# $Id: unpack.pl,v 1.1.2.1 2002/04/25 08:47:05 daveewart Exp $

########################################################################
#                                                                      #
# Unpack - a script to extract contents of tarballs to fixed location  #
# Copyright (C)2001-2002 Dave Ewart (davee@sungate.co.uk)              #
#                                                                      #
########################################################################
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License as published by #
# the Free Software Foundation; either version 2 of the License, or    #
# (at your option) any later version.                                  #
#                                                                      #
# This program is distributed in the hope that it will be useful,      #
# but WITHOUT ANY WARRANTY; without even the implied warranty of       #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        #
# GNU General Public License for more details.                         #
#                                                                      #
# You should have received a cop of the GNU General Public License     #
# along with this program; if not, write to the Free Software          #
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            #
#                                                                      #
########################################################################

use strict;

my $copyyear         = '2001-2002';
my $author           = 'Dave Ewart';
my $author_email     = 'davee@sungate.co.uk';
my $version          = '1.0.1';
my $version_date_raw = '$Date: 2002/04/25 08:47:05 $';
my $version_date     =
  join ('.', substr($version_date_raw, 15, 2), substr($version_date_raw, 12, 2),
  substr($version_date_raw, 7, 4));

print "(C)$copyyear $author, $author_email\n";
print "Unpack version $version ($version_date)\n";
print "\n";

# Look for ~/.unpackrc in home directory and use extraction
# directory found there in preference to other defaults
# Otherwise, use $HOME/src or /tmp as final alternative 
# if HOME does not exist

my $extract_dir;
my $extract_subdir = 'src';
if (defined $ENV{HOME}) {
    if (open(UNPACKRC, "<$ENV{HOME}/.unpackrc")) {
        my ($rc_setting, $rc_value);
        while (<UNPACKRC>) {
            next if /^#/;
            chop;
            ($rc_setting, $rc_value) = split('=');
            if ($rc_setting eq 'DIR') {
                $extract_dir=$rc_value;
            }
            else {
                chop;
                print "Unknown configuration option: $_ ... ignored\n";
            }
        }
        close(UNPACKRC);
    }
    if (!defined $extract_dir) {
        $extract_dir = join ('/', $ENV{HOME}, $extract_subdir);
    }
}
else {
    $extract_dir = '/tmp';
}

# Sanity check - does the extraction directory exist

opendir(EXTRACT, $extract_dir) || die "Cannot open directory $extract_dir\n";
closedir(EXTRACT);

my $nargs = @ARGV;
if ($nargs == 0) {
    print "Usage:                unpack TARBALL\n";
    print "Files extracted to:   $extract_dir\n";
    print "Known archive types:  *.tar, *.tar.gz, *.tgz, *.tar.bz2\n";
    print "\n";
    exit(0);
}

my $target_tarball = $ARGV[0];
print STDERR "Identifying compression type ... ";

# Choose appropriate option for 'tar' based on file extension
# TODO make this more sophisticated
my ($tar_option, $archive_name);

# .tar.gz and .tgz - extracted using: tar -zxf
if ($target_tarball =~ /\.tar\.gz$/ || $target_tarball =~ /\.tgz$/) {
    $tar_option   = 'z';
    $archive_name = $`;
    print " gzipped tarball: ";
}
# .tar.bz2 - extracted using: tar -Ixf
elsif ($target_tarball =~ /\.tar\.bz2$/) {
    $tar_option   = '-bzip2 -';
    $archive_name = $`;
    print " bzip2 tarball: ";
}
# .tar (uncompressed) - extracted using: tar -xf
elsif ($target_tarball =~ /\.tar$/) {
    $tar_option   = '';
    $archive_name = $`;
    print " uncompressed tar archive: ";
}
else {
    print "cannot identify archive type.\n";
    exit(0);
}
print "OK\n";

# Strip leading path from archive file/path
# e.g. 'unpack ~/temp/project-1.2.tar.gz' 
# returns 'project-1.2' into $archive_name
$archive_name = substr($archive_name, 1 + rindex($archive_name, '/'));

print STDERR "Checking that archive extracts to its own directory ... ";

# If all files in the archive have the same top-level paths, then use this
# as the extraction path (i.e. extract to /home/user/src/toplevel/...
#
# If there are no paths, or files appear to have different top-level paths
# then use the archive name as the toplevel path

my $archive_ok    = 1;
my $any_paths     = 0;
my $old_mypath    = "";
my $tar_t_options = "-${tar_option}tf";
open(TEMP, "tar $tar_t_options $target_tarball|") || die "Cannot open archive";

my ($file, $mypath);
while (<TEMP>) {
    $file = $_;
    if ($file =~ /\//) {
        $any_paths = 1;
        $mypath    = $`;
        if ($old_mypath ne "" && $old_mypath ne $mypath) {
            $archive_ok = 0;
        }
    }
    else {
        $mypath = "NOPATH";
    }
    $old_mypath = $mypath;
}
close(TEMP);

if ($any_paths == 0) {
    $archive_ok = 0;
}
if ($archive_ok == 1) {
    print STDERR "Yes.\n";
}
else {
    print STDERR "No.\nCreating path $extract_dir/$archive_name ... ";
    # TODO Should examine return values for each system call
    system("mkdir -p $extract_dir/$archive_name");
    print STDERR "OK.\n";
    $extract_dir = "$extract_dir/$archive_name";
}

print STDERR "Unpacking $target_tarball to $extract_dir ... ";

# TODO Should examine return values for each system call
system("tar -${tar_option}xf $target_tarball --directory $extract_dir");
print STDERR "OK\n";

